function [ imfilt ] = FDWF(im,beta,blockSize)
% Performs frequency domain Wiener filtering on an image layer using 
% techniques proposed by Sari et. al.
%
% Usage: FDWF(image,beta,block size)
%   image - MxN grayscale image
%   beta - division ratio used for thresholding
%   block size - size of computation block used in filter, typically 2^n
%   Note: image should already have FFT applied.
tic;
% Calculate power and log power spectrum
Pd = abs(im).^2;
Xd = log10(Pd);

blockAverage = zeros(blockSize,blockSize);

% Set S and T block size
[height width layers] = size(im);
S = height/blockSize;
T = width/blockSize;

% Calculate mean of each block
for n = 1:blockSize
    for m = 1:blockSize
        blockAverage(m,n) = mean(mean(Xd((m-1)*S+1:m*S,(n-1)*T+1:n*T)));
    end
end

% Calculate median and minimum
blockMedian = median(blockAverage(:));
blockMin = min(blockAverage(:));

% Threshold
Th = blockMedian*(beta/100)+blockMin;

% Create noise power spectrum
highAvg = mean([blockAverage(1,1),blockAverage(blockSize,1),...
    blockAverage(1,blockSize),blockAverage(blockSize,blockSize)]);
Pn = zeros(height,width);
for m = 1:blockSize
    for n = 1:blockSize
        if(blockAverage(m,n) <= Th)
            for a = (m-1)*S+1:(m*S)
                for b = (n-1)*T+1:(n*T)
                    Pn(a,b) = Pd(a,b);
                end
            end
        else
            for a = (m-1)*S+1:(m*S)
                for b = (n-1)*T+1:(n*T)
                    Pn(a,b) = highAvg;
                end
            end
        end
    end
end

% Create signal power spectrum
Ps = zeros(height,width);
for m = 1:blockSize
    for n = 1:blockSize
        if(blockAverage(m,n) > Th)
            for a = (m-1)*S+1:(m*S)
                for b = (n-1)*T+1:(n*T)
                    Ps(a,b) = Pd(a,b);
                end
            end
        else
            for a = (m-1)*S+1:(m*S)
                for b = (n-1)*T+1:(n*T)
                    Ps(a,b) = 0;
                end
            end
        end
    end
end

% Create Wiener filter
w = Ps./(Ps+Pn);
imfilt = w;
toc

end

